int np1;
int np2;
int np3;
int totalNp;
float rad1;
float rad2;
float rad3;

disk[] particles;

//float springConst = .008;
float springConst = 0.1;
float cutoff = 10;
float interactionConstant = .1;
float diskMuStatic = 5;
float diskMuStokes = .1;
float wallMuStatic = 50;
float wallMuStokes = 50;

int h = 400;
int w = 600;
int bw = 10; // border width
int inputBoxWidth = 200;

float phi; //packing fraction
float setupPhi = 0;
String phiText = "";

int frameCounter = 0;
boolean possibleOverlap; // for looping to fix overlap - not currently used
boolean onMenuScreen = true;
boolean gameMode; // true for game mode, false for sandbox mode // need to have this set by user
boolean started = false;

float movesPerFrame = 8;
float dt = 1/movesPerFrame;

int cMode = 1;

button gameButton;
button sandboxButton;

slider rad1Slider;
slider rad2Slider;
slider rad3Slider;
slider np1Slider;
slider np2Slider;
slider np3Slider;
button startButton;

button resetButton;
button sizeColorButton;
button pressureColorButton;

hWall bottomWall = new hWall(h-bw,1);
hWall topWall = new hWall(bw,-1);
vWall rightWall = new vWall(w-bw,-1);
//vWall leftWall = new vWall(bw,1); // static wall
vSpringWall leftWall = new vSpringWall(bw,1); // spring wall




void setup(){
  
  size(w+inputBoxWidth,h);
  frameRate(30);
  //frameRate(1);

  background(50);
  
  menuSetup();
  
  topWall.display();
  bottomWall.display();
  rightWall.display();
  drawTicks();
  leftWall.display();
  
}


void menuSetup() {
  gameButton = new button(w/3,h/2,80,20,"game mode");
  sandboxButton = new button(2*w/3,h/2,100,20,"sandbox mode");
  gameButton.display();
  sandboxButton.display();
}


void gameSetup() {
  np1 = 20;
  np2 = 20;
  np3 = 20;
  totalNp = np1+np2+np3;
  rad1 = 29;
  rad2 = 21.5;
  rad3 = 17.5;
  
  float phiInit = ( np1*PI*pow(rad1,2) + np2*PI*pow(rad2,2) + np3*PI*pow(rad3,2) ) / ( (w-2*bw)*(h-2*bw) );
  phiText = "packing fraction: " + str(phiInit);
  
  particleSetup();
  
  started = true;
  
}


void sandboxSetup() {
  np1 = 0;
  np2 = 0;
  np3 = 0;
  totalNp = 0;
  rad1 = 10;
  rad2 = 10;
  rad3 = 10;
  
  rad1Slider = new slider(w+20, 100, 45, 25);
  np1Slider = new slider(w+20, 100, 95);
  rad2Slider = new slider(w+20, 100, 145, 25);
  np2Slider = new slider(w+20, 100, 195);
  rad3Slider = new slider(w+20, 100, 245, 25);
  np3Slider = new slider(w+20, 100, 295);
  startButton = new button(w+80, h-80, 40, 20, "start");
  
  float setupPhi = 0;
  phiText = "packing fraction: " + str(setupPhi);
  
  rad1Slider.display();
  np1Slider.display();
  rad2Slider.display();
  np2Slider.display();
  rad3Slider.display();
  np3Slider.display();
  startButton.display();
    
}


void draw() {
  background(50);
  
  if (onMenuScreen) {
    menuDraw();
  } else if (!started && !gameMode) {
    sandboxSetupDraw();
  }
  
  if (started) {
    for (int i=0; i<4; i++) {
      simulationStep();
    }
    simulationDraw();
  }
  
}


void menuDraw() {
  
  gameButton.display();
  sandboxButton.display();
  
}


void sandboxSetupDraw() { //rename this...
  
  ellipseMode(CENTER);
  textAlign(CENTER);
  
  setSliderValues();
  
  rad1Slider.display();
  fill(255);
  ellipse(w+160, rad1Slider.y, 2*rad1, 2*rad1);
  //fill(0);
  //text(rad1, w+160, rad1Slider.y+10);
  
  np1Slider.display();
  fill(255);
  text(np1, w+160, np1Slider.y);
  
  rad2Slider.display();
  fill(255);
  ellipse(w+160, rad2Slider.y, 2*rad2, 2*rad2);
  
  np2Slider.display();
  fill(255);
  text(np2, w+160, np2Slider.y);
  
  rad3Slider.display();
  fill(255);
  ellipse(w+160, rad3Slider.y, 2*rad3, 2*rad3);
  
  np3Slider.display();
  fill(255);
  text(np3, w+160, np3Slider.y);
  
  startButton.display();
  textAlign(CORNER);
  fill(255);
  text(phiText, w+10, h-20);
  
  topWall.display();
  bottomWall.display();
  rightWall.display();
  drawTicks();
  leftWall.display();
  
  resetButton.display();
  
}

void simulationStep() {
  
    for (int i=0; i<totalNp; i++) {
    particles[i].resetForce();
  }
  leftWall.resetForce();

  applyInterparticleForces();
  applyParticleWallForces();
  leftWall.addForce( springConst*(w-leftWall.x1) );
  
  for (int i=0; i<totalNp; i++) {
    particles[i].integrateVerlet();
  }
  leftWall.integrateVerlet();
  
  // try to make sure all collisions are fixed. this is getting stuck in a loop
  /*possibleOverlap = true;
  while (possibleOverlap) {
    possibleOverlap = particleCollisions();
  }*/
  particleCollisions();
  fixedWallCollisions();
  if (totalNp>0) {
    springWallCollisions();
  }
  
  for (int i=0; i<totalNp; i++) {
    particles[i].addFriction();
  }
  
  leftWall.addFriction();

  
}

void simulationDraw() {

  for (int i=0; i<totalNp; i++) {
    particles[i].display();
  }
  
  topWall.display();
  bottomWall.display();
  rightWall.display();
  drawTicks();
  leftWall.display();
  
  if ( frameCounter%30 == 0 ) {
    phi = packingFraction();
    phiText = "packing fraction: " + str(phi);
  }
  textAlign(CORNER);
  text(phiText, w+10, h-20);
  
  resetButton.display();
  
  textAlign(CENTER);
  fill(255);
  text("color by:", w+100, h/4);
  sizeColorButton.display();
  pressureColorButton.display();
  
  frameCounter++;
  
}


void mousePressed() {
  float rm;
  
  if (onMenuScreen) {
    menuMousePressed();
  } else if ( !gameMode && !started ) {
    sandboxSetupMousePressed();
  } else if (started) {
    simulationMousePressed();
  }
  
}


void menuMousePressed() {
  
  if ( gameButton.mouseOnQ() ) {
    onMenuScreen = false;
    gameMode = true;
    gameSetup();
    resetButton = new button(0, h-20, 60, 20, "reset");
    sizeColorButton = new button (w+60, 3*h/8, 80, 20, "size");
    pressureColorButton = new button (w+60, h/2, 80, 20, "pressure");
  }
  
  if ( sandboxButton.mouseOnQ() ) {
    onMenuScreen = false;
    gameMode = false;
    sandboxSetup();
    resetButton = new button(0, h-20, 60, 20, "reset");
    sizeColorButton = new button (w+60, 3*h/8, 80, 20, "size");
    pressureColorButton = new button (w+60, h/2, 80, 20, "pressure");
  }
  
}


void sandboxSetupMousePressed() {
     
  // check if the mouse is over a slider, and if so start dragging
  if (rad1Slider.mouseOnQ()) {
    rad1Slider.lock(mouseX);
  }
  
  if (np1Slider.mouseOnQ()) {
    np1Slider.lock(mouseX);
  }
  
  if (rad2Slider.mouseOnQ()) {
    rad2Slider.lock(mouseX);
  }
  
  if (np2Slider.mouseOnQ()) {
    np2Slider.lock(mouseX);
  }
  
  if (rad3Slider.mouseOnQ()) {
    rad3Slider.lock(mouseX);
  }
  
  if (np3Slider.mouseOnQ()) {
    np3Slider.lock(mouseX);
  }
  
  if (startButton.mouseOnQ()) {
    started = true;
    
    setSliderValues();
        
    float phiInit = ( np1*PI*pow(rad1,2) + np2*PI*pow(rad2,2) + np3*PI*pow(rad3,2) ) / ( (w-2*bw)*(h-2*bw) );
    phiText = "packing fraction: " + str(phiInit);
    
    particleSetup();
  }
  
  if ( resetButton.mouseOnQ() ) {
    resetGame();
  }
  
  if ( sizeColorButton.mouseOnQ() ) {
    cMode = 1;
  }
  
  if ( pressureColorButton.mouseOnQ() ) {
    cMode = 2;
  }
  
}


void simulationMousePressed() {
  float rm;
  
  // check if the mouse is over any particle, and if so remove it
  clickLoop:
  for (int i=0; i<totalNp; i++) {
    rm = distance(mouseX, mouseY, particles[i].x1, particles[i].y1);
    if (rm < particles[i].rad) {
      totalNp--;
      /*if (totalNp == 0) {
        println("no more particles!");
        exit();
      }*/
      for (int j=i; j<totalNp; j++) {
        particles[j] = particles[j+1];
      }
      break clickLoop;
    }
  }
  
  if ( resetButton.mouseOnQ() ) {
    resetGame();
  }
  
  if ( sizeColorButton.mouseOnQ() ) {
    cMode = 1;
  }
  
  if ( pressureColorButton.mouseOnQ() ) {
    cMode = 2;
  }
  
}

void resetGame() {
  onMenuScreen = true;
  started = false;
  leftWall.resetPosition(bw);
}
  
  


void mouseDragged() {
 
  if (!gameMode && !started) {
    if (rad1Slider.locked) {
      rad1Slider.dragTo(mouseX);
    }
    if (rad2Slider.locked) {
      rad2Slider.dragTo(mouseX);
    }
    if (rad3Slider.locked) {
      rad3Slider.dragTo(mouseX);
    }
    if (np1Slider.locked) {
      np1Slider.dragTo(mouseX);
    }
    if (np2Slider.locked) {
      np2Slider.dragTo(mouseX);
    }
    if (np3Slider.locked) {
      np3Slider.dragTo(mouseX);
    }
    
    setSliderValues();
  }
  
  float setupPhi = ( np1*PI*pow(rad1,2) + np2*PI*pow(rad2,2) + np3*PI*pow(rad3,2) ) / ( (w-2*bw)*(h-2*bw) );
  phiText = "packing fraction: " + str(setupPhi);
}


void mouseReleased() {
  
  if (!gameMode && !started) {
    rad1Slider.unlock();
    rad2Slider.unlock();
    rad3Slider.unlock();
    np1Slider.unlock();
    np2Slider.unlock();
    np3Slider.unlock();
  }
}

void setSliderValues() {
  

  rad1 = 0 + 30*(rad1Slider.knobX/rad1Slider.dx);
  rad2 = 0 + 30*(rad2Slider.knobX/rad2Slider.dx);
  rad3 = 0 + 30*(rad3Slider.knobX/rad3Slider.dx);
  
  np1 = int(0 + 100*(np1Slider.knobX/np1Slider.dx));
  np2 = int(0 + 100*(np2Slider.knobX/np2Slider.dx));
  np3 = int(0 + 100*(np3Slider.knobX/np3Slider.dx));
  totalNp = np1+np2+np3;
  
}

void particleSetup() {
  //use random sequential adsorption to initialize particles
  float xtemp, ytemp, xcomp, ycomp, radTemp;
  boolean overlap;
  int attempts = 0;
  
  particles = new disk[totalNp];
  
  placementLoop:
  for (int i=0; i<totalNp; i++) {
    
    if ( i < np1 ) {
      radTemp = rad1;
    } else if ( i < np1 + np2 ) {
      radTemp = rad2;
    } else {
      radTemp = rad3;
    }
    
    attempts = 0;
    
    do {
      attempts++;
      if ( attempts > 10000 ) {
        textAlign(CENTER,CENTER);
        fill(255);
        text("having trouble placing that many particles of the chosen size(s). Try fewer/smaller particles.", w/2,  h/2);
        started = false;
        break placementLoop;
      }
      overlap = false;
      xtemp = random(radTemp+bw,w-bw-radTemp);
      ytemp = random(radTemp+bw,h-bw-radTemp);
      // check for particle overlap
      for (int j=0; j<i; j++) {
        //xcomp=particles[j].getX();
        //ycomp=particles[j].getY();
        xcomp=particles[j].x1;
        ycomp=particles[j].y1;
        if ( distance(xtemp, ytemp, xcomp, ycomp) < radTemp+particles[j].rad ) {
          overlap = true;
        }
      }
    } while( overlap == true );
    particles[i] = new disk(radTemp, xtemp, ytemp);
  }
}


float distance(float x1, float y1, float x2, float y2) {
  return sqrt(pow( (x1-x2), 2 ) + pow( (y1-y2), 2 ));
}


/*
float particleForce(float r) {
  float a = interactionConstant; // hardness
  float rMax = 3*rad; // cutoff
  float r0 = 2*rad; // closest approach of particle centers
  float offset = a/pow(rMax,2); //Make force 0 at cutoff
  
  if (r < r0) {
    //return coreForce;
    return 0.0;
  } else if (r > rMax) {
    return 0.0;
  } else {  
    //return a/(r-r0); // based on potential -a*Log(r-r0)
    //return a/pow(r,2);
    return a/pow(r,2) - offset;
    //return 0;
  }
}
*/

/*
float wallForce(float r) {
  float a = interactionConstant; // hardness
  float rMax = 2*rad; // cutoff
  float r0 = rad;
  float offset = a/pow(rMax,2); //Make force 0 at cutoff
  
  if (r < r0) {
    //return coreForce;
    return 0;
  } else if (r > rMax) {
    return 0.0;
  } else {  
    //return a/(r-r0); // based on potential -a*Log(r-r0)
    //return a/pow(r,2);
    return a/pow(r,2) - offset;
    //return 0;
  }
}
*/


float force(float r) {
  // use distance between particle/wall edges - not from centers
  float a = interactionConstant; // hardness
  float rMax = cutoff; // cutoff
  float offset = a/pow(rMax,2); //Make force 0 at cutoff
  
  if (r > rMax) {
    return 0.0;
  } else {  
    //return a/(r-r0); // based on potential -a*Log(r-r0)
    //return a/pow(r,2);
    //return a/pow(r,2) - offset;
    //return 0;
    //return a; // constant force if within cutoff
    //return a-(a/rMax)*r;
    //return (a/pow(rMax,2))*pow(r-rMax,2);
    return (a/pow(rMax,4))*pow(r-rMax,4);
  }
}


void applyInterparticleForces() {
  float rp, fp, deltax, deltay;
  for (int i=0; i<totalNp; i++) {
    for (int j=0; j<i; j++) {
      deltax = particles[j].x1 - particles[i].x1;
      deltay = particles[j].y1 - particles[i].y1;
      rp = sqrt( pow(deltax,2) + pow(deltay,2) );
      fp = force(rp - particles[j].rad - particles[i].rad);
      
      particles[i].addForce(-fp*deltax/rp, -fp*deltay/rp);
      particles[j].addForce(fp*deltax/rp, fp*deltay/rp);
    }
  }
  
}


void applyParticleWallForces() {
  float rt, rb, rr, rl, ft, fb, fr, fl;
  
  for (int i=0; i<totalNp; i++) {
    rt = abs( topWall.y - particles[i].y1 );
    ft = force(rt - particles[i].rad);
    particles[i].addForce(0,-ft);
    
    rb = abs( bottomWall.y - particles[i].y1 );
    fb = force(rb - particles[i].rad);
    particles[i].addForce(0,fb);
    
    rr = abs( rightWall.x - particles[i].x1 );
    fr = force(rr - particles[i].rad);
    particles[i].addForce(-fr,0);
    
    rl = abs( leftWall.x1 - particles[i].x1 );
    fl = force(rl - particles[i].rad);
    particles[i].addForce(fl,0);
    leftWall.addForce(-fl);
  }
  
}


boolean particleCollisions() {
  //simply undoes overlaps by shifting particles so that they are just touching
  // returns true if any collisions were fixed
  boolean fixed = false;
  float r, deltax, deltay, radDistance;
  
  for (int i=0; i<totalNp; i++) {
    for (int j=0; j<i; j++) {
      r = distance(particles[i].x1, particles[i].y1, particles[j].x1, particles[j].y1);
      deltax = particles[j].x1 - particles[i].x1;
      deltay = particles[j].y1 - particles[i].y1;
      
      radDistance = particles[i].rad + particles[j].rad;
      if ( r < radDistance ) {
        fixed = true;
        particles[i].setX(particles[i].x1 - 0.5*(radDistance-r)*deltax/r);
        particles[i].setY(particles[i].y1 - 0.5*(radDistance-r)*deltay/r);
        particles[j].setX(particles[j].x1 + 0.5*(radDistance-r)*deltax/r);
        particles[j].setY(particles[j].y1 + 0.5*(radDistance-r)*deltay/r);
      }
      
    }
  }
  
  return fixed;
  
}


void fixedWallCollisions() {
  float rt, rb, rr, rl;
  
  for (int i=0; i<totalNp; i++) {
    
    rt = abs(topWall.y - particles[i].y1);
    if (rt < particles[i].rad) {
      particles[i].setY(topWall.y + particles[i].rad);
    }
    
    rb = abs(particles[i].y1 - bottomWall.y);
    if (rb < particles[i].rad) {
      particles[i].setY(bottomWall.y - particles[i].rad);
    }
    
    rr = abs(rightWall.x - particles[i].x1);
    if (rr < particles[i].rad) {
      particles[i].setX(rightWall.x - particles[i].rad);
    }
    
  }
}


void springWallCollisions() {
  float leftMostX, rOverlap;
  int leftMostIndex;
  
  leftMostIndex = -1;
  leftMostX = w;
  
  for (int i=0; i<totalNp; i++) {
    if ( particles[i].x1-particles[i].rad < leftMostX ) {
      leftMostIndex = i;
      leftMostX = particles[i].x1-particles[i].rad;
    }
  }
  
  rOverlap = leftWall.x1 - leftMostX; // sometimes gives ArrayIndexOutOfBoundsException (w/ log potential)
 // - all particles x positions give NaN, some sort of instability
 // This seems to happen with divergent forces
  if (rOverlap > 0) {
    particles[leftMostIndex].setX(particles[leftMostIndex].x1 + rOverlap/2);
    leftWall.setX(leftWall.x1 - rOverlap/2);
  }
}


float packingFraction() {
  float diskAreaSum = 0;
  float boardArea = (w-bw-leftWall.x1)*(h-2*bw);
  
  for (int i=0; i<totalNp; i++) {
    diskAreaSum += PI*pow(particles[i].rad,2);
  }
  
  return diskAreaSum/boardArea;
  
}

void drawTicks() {
  for (int i=0; i<60; i++) {
    line(i*10,0,i*10,10);
  }
}
class button {
  int x, y, dx, dy;
  String label;
  
  button(int x_, int y_, int dx_, int dy_, String label_) {
    x = x_;
    y = y_;
    dx = dx_;
    dy = dy_;
    label = label_;
  }
  
  
  void display() {
    
    rectMode(CORNER);
    fill(255);
    rect(x, y, dx, dy);
    fill(0);
    textAlign(CENTER,CENTER);
    text(label, x+dx/2, y+dy/2);
    
  }
  
  
  boolean mouseOnQ() {
    
    if (mouseX > x && mouseX < x+dx && mouseY > y && mouseY < y+dy) {
      return true;
    } else {
      return false;
    }
    
  }
  
}
class disk {
  float rad, x1, y1, x0, y0, vx, vy, fx, fy, pressure;
  color c = color(100,100,100);
//  float mu = .004; // coeff of friction
  float muStatic = diskMuStatic;
  float muStokes = diskMuStokes;
  //float muKinetic = 0.05;
  float coordNum = 0;
  
  disk(float rad_, float x_, float y_) {
    x0 = x_;
    y0 = y_;
    x1 = x_;
    y1 = y_;
    rad = rad_;
    vx = 0;
    vy = 0;
    fx = 0;
    fy = 0;
    pressure = 0;
  }
  
  void display() {
    ellipseMode(CENTER);
    switch(cMode) {
      case 1: //based on radius
        fill(rad/40*255);
        break;
      case 2:
        fill(pressure/10*255);
        break;
      case 3:
        fill(255);
        break;
    }
      
    ellipse(x1,y1,rad*2,rad*2);
  }
  
  void integrateEuler() {
    vx += fx*dt;
    vy += fy*dt;
    x1 += vx*dt;
    y1 += vy*dt;
  }
  
  void integrateVerlet() {
    float vMag, fMag, xHold, yHold;
    
    vMag = sqrt(pow(vx,2)+pow(vy,2));
    fMag = sqrt(pow(fx,2)+pow(fy,2));
    
    if (vMag > 0 || fMag > muStatic) {
      xHold = x1;
      yHold = y1;
      x1 = 2*x1 - x0 + fx*pow(dt,2);
      y1 = 2*y1 - y0 + fy*pow(dt,2);
      x0 = xHold;
      y0 = yHold;
      vx = (x1-x0)/dt;
      vy = (y1-y0)/dt;
    }
  }
  
  void setX(float x_) {
    x1 = x_;
  }
  
  void setY(float y_) {
    y1 = y_;
  }
  
  void resetForce() {
    fx = 0;
    fy = 0;
    pressure = 0;
  }
  
  void addForce(float dfx_, float dfy_) {
    fx += dfx_;
    fy += dfy_;
    pressure += sqrt( pow(fx,2) + pow(fy,2) );
  }
  
  void addFriction() {
    float vMag, frx, fry;
    
    vMag = sqrt(pow(vx,2)+pow(vy,2));
    
    if ( vMag > 0 ) {
      //note: fr and v will have same sign
      //frx = muKinetic*vx/vMag;
      //fry = muKinetic*vy/vMag;
      frx = muStokes*vx;
      fry = muStokes*vy;
    } else {
      frx = 0;
      fry = 0;
    }
    
    // dont overshoot: if friction would reverse direction, make v 0
    if ( abs(frx) > abs(vx) || abs(fry) > abs(vy) ) {
      x1 -= vx*dt;
      y1 -= vy*dt;
    } else {
      x1 -= frx*pow(dt,2);
      y1 -= fry*pow(dt,2);
    }
    
    // take adjusted velocity
    vx = (x1-x0)/dt;
    vy = (y1-y0)/dt;
    
  }
  
  
}
class hWall {
  //a horizontal wall, specified by its veritcal position and orientation of normal vector (up: dir=1, down: dir=-1)
  float y;
  int dir;
  
  hWall(float y_, int dir_) {
    y = y_;
    dir = dir_;
    if ( dir !=1 & dir != -1 ) {
      println("dir must be either 1 or -1");
      exit();
    }
  }
  
  void display() {
    rectMode(CORNERS);
    fill(255);
    if (dir>0) {
      rect(0,y,w,h);
    } else {
      rect(0,y,w,0);
    }
  }
  
}
class slider {
  int x, dx, y; // horizontal position of left side; width; vertical position
  float knobX; // horizontal position of slider knob relatie to left side
  float rad = 5; // standard knob radius
  boolean locked = false;
  float offset = 0;
 

  slider(int x_, int dx_, int y_) {
    x = x_;
    dx = dx_;
    y = y_;
    knobX = 0;
  }

 
  slider(int x_, int dx_, int y_, float knobX_) {
    x = x_;
    dx = dx_;
    y = y_;
    knobX = knobX_;
  }

  
  void display() {
    line(x, y, x+dx, y);
    ellipseMode(CENTER);
    fill(255);
    ellipse(x+knobX, y, rad*2, rad*2);
  }
  

  boolean mouseOnQ() {
    float rm;
    
    rm = sqrt( pow( mouseX - (x+knobX), 2) + pow(mouseY-y, 2) );
    if (rm < rad) {
      return true;
    } else {
      return false;
    }
  }

  
  void lock(float mx_) {
    //take current mouse x position for dragging offset
    locked = true;
    offset = mx_ - (x+knobX);
  }
  
  
  void dragTo(float mx_) {
    knobX = (mx_-x) + offset;
    if (knobX < 0) {
      knobX = 0;
    }
    if (knobX > dx) {
      knobX = dx;
    }
  }

  
  void unlock() {
    locked = false;
    offset = 0;
  }
  
  
}
class vSpringWall {
  //a vertical wall, specified by its horizontal position and orientation (right: dir=1, left: dir=-1)
  float x1, x0, vx, fx;
  float mu = .01;
  float muStatic = wallMuStatic;
  float muStokes = wallMuStokes;
  int dir;
  
  vSpringWall(float x_, int dir_) {
    x0 = x_;
    x1 = x_;
    vx = 0;
    dir = dir_;
    if ( dir != 1 & dir != -1 ) {
      println("dir must be either 1 or -1");
      exit();
    }
  }
  
  void display() {
    //add error handling for dir=0 (or put in constructor)
    rectMode(CORNERS);
    fill(255);
    if (dir>0) {
      rect(0,0,x1,h);
    } else {
      rect(x1,0,w,h);
    }
  }
  
  void integrateEuler() {
    vx += fx*dt;
    x1 += vx*dt;
  }
  
  void integrateVerlet() {
    float vMag, fMag, xHold;
    
    vMag = abs(vx);
    fMag = abs(fx);
    
    if (vMag > 0 || fMag > muStatic) {
      xHold = x1;
      x1 = 2*x1 - x0 + fx*pow(dt,2);
      x0 = xHold;
      vx = (x1-x0)*dt;
    }
    
  }
  
  void setX(float x_) {
    x1 = x_;
  }
  
  void resetPosition(float x_) {
    x0 = x_;
    x1 = x_;
    vx = 0;
  }
  
  void resetForce() {
    fx = 0;
  }
  
  void addForce(float dfx_) {
    fx += dfx_;
  }
  
  void addFriction() {
    float vMag, frx;
    
    vMag = abs(vx);
    
    if ( vMag > 0 ) {
      //note: fr and v will have same sign
      //frx = muKinetic*vx/vMag;
      frx = muStokes*vx;
    } else {
      frx = 0;
    }
    
    // dont overshoot: if friction would reverse direction, make v 0
    if ( abs(frx) > abs(vx) ) {
      x1 -= vx*dt;
    } else {
      x1 -= frx*pow(dt,2);
    }
    
    // take adjusted velocity
    vx = (x1-x0)/dt;
    
  }
  
}
class vWall {
  //a vertical wall, specified by its horizontal position and orientation (right: dir=1, left: dir=-1)
  float x;
  int dir;
  
  vWall(float x_, int dir_) {
    x = x_;
    dir = dir_;
    if ( dir != 1 & dir != -1 ) {
      println("dir must be either 1 or -1");
      exit();
    }
  }
  
  void display() {
    //add error handling for dir=0 (or put in constructor)
    rectMode(CORNERS);
    fill(255);
    if (dir>0) {
      rect(0,0,x,h);
    } else {
      rect(x,0,w,h);
    }
  }
  
}

